@@ -1,9 +1,9 @@
 use std::env;
 
 use chrono::Utc;
+use serde::{Deserialize, Serialize};
 
 use atuin_common::utils::uuid_v4;
-use serde::{Deserialize, Serialize};
 
 // Any new fields MUST be Optional<>!
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq, sqlx::FromRow)]
@@ -45,4 +45,8 @@ impl History {
             hostname,
         }
     }
+
+    pub fn success(&self) -> bool {
+        self.exit == 0 || self.duration == -1
+    }
 }
