@@ -3,12 +3,21 @@ use std::convert::TryInto;
 use chrono::prelude::*;
 use eyre::Result;
 
-use atuin_common::{api::AddHistoryRequest, utils::hash_str};
+use atuin_common::api::AddHistoryRequest;
 
-use crate::api_client;
-use crate::database::Database;
-use crate::encryption::{encrypt, load_encoded_key, load_key};
-use crate::settings::{Settings, HISTORY_PAGE_SIZE};
+use crate::{
+    api_client,
+    database::Database,
+    encryption::{encrypt, load_encoded_key, load_key},
+    settings::{Settings, HISTORY_PAGE_SIZE},
+};
+
+pub fn hash_str(string: &str) -> String {
+    use sha2::{Digest, Sha256};
+    let mut hasher = Sha256::new();
+    hasher.update(string.as_bytes());
+    hex::encode(hasher.finalize())
+}
 
 // Currently sync is kinda naive, and basically just pages backwards through
 // history. This means newly added stuff shows up properly! We also just use
