@@ -1,9 +1,12 @@
+use std::{env, io::stdout, ops::Sub, time::Duration};
+
 use chrono::Utc;
 use clap::Parser;
 use eyre::Result;
-use std::env;
-use std::{io::stdout, ops::Sub, time::Duration};
-use termion::{event::Key, input::MouseTerminal, raw::IntoRawMode, screen::AlternateScreen};
+use termion::{
+    event::Event as TermEvent, event::Key, event::MouseButton, event::MouseEvent,
+    input::MouseTerminal, raw::IntoRawMode, screen::AlternateScreen,
+};
 use tui::{
     backend::{Backend, TermionBackend},
     layout::{Alignment, Constraint, Corner, Direction, Layout},
@@ -22,7 +25,10 @@ use atuin_client::{
     settings::{FilterMode, SearchMode, Settings},
 };
 
-use super::event::{Event, Events};
+use super::{
+    event::{Event, Events},
+    history::ListMode,
+};
 
 const VERSION: &str = env!("CARGO_PKG_VERSION");
 
@@ -52,6 +58,10 @@ pub struct Cmd {
     #[clap(long)]
     after: Option<String>,
 
+    /// How many entries to return at most
+    #[clap(long)]
+    limit: Option<i64>,
+
     /// Open interactive search UI
     #[clap(long, short)]
     interactive: bool,
@@ -68,32 +78,43 @@ pub struct Cmd {
 }
 
 impl Cmd {
-    pub async fn run(
-        self,
-        db: &mut (impl Database + Send + Sync),
-        settings: &Settings,
-    ) -> Result<()> {
-        run(
-            settings,
-            self.cwd,
-            self.exit,
-            self.interactive,
-            self.human,
-            self.exclude_exit,
-            self.exclude_cwd,
-            self.before,
-            self.after,
-            self.cmd_only,
-            &self.query,
-            db,
-        )
-        .await
+    pub async fn run(self, db: &mut impl Database, settings: &Settings) -> Result<()> {
+        if self.interactive {
+            let item = select_history(
+                &self.query,
+                settings.search_mode,
+                settings.filter_mode,
+                settings.style,
+                db,
+            )
+            .await?;
+            eprintln!("{}", item);
+        } else {
+            let list_mode = ListMode::from_flags(self.human, self.cmd_only);
+            run_non_interactive(
+                settings,
+                list_mode,
+                self.cwd,
+                self.exit,
+                self.exclude_exit,
+                self.exclude_cwd,
+                self.before,
+                self.after,
+                self.limit,
+                &self.query,
+                db,
+            )
+            .await?;
+        };
+        Ok(())
     }
 }
 
 struct State {
     input: String,
 
+    cursor_index: usize,
+
     filter_mode: FilterMode,
 
     results: Vec<History>,
@@ -196,7 +217,7 @@ impl State {
 
                 let duration = Span::styled(
                     duration,
-                    Style::default().fg(if m.exit == 0 || m.duration == -1 {
+                    Style::default().fg(if m.success() {
                         Color::Green
                     } else {
                         Color::Red
@@ -237,7 +258,7 @@ impl State {
 async fn query_results(
     app: &mut State,
     search_mode: SearchMode,
-    db: &mut (impl Database + Send + Sync),
+    db: &mut impl Database,
 ) -> Result<()> {
     let results = match app.input.as_str() {
         "" => {
@@ -261,15 +282,40 @@ async fn query_results(
     Ok(())
 }
 
+fn get_input_prefix(app: &mut State, i: usize) -> String {
+    return app.input.chars().take(i).collect();
+}
+fn get_input_suffix(app: &mut State, i: usize) -> String {
+    return app.input.chars().skip(i).collect();
+}
+
+fn insert_char_into_input(app: &mut State, i: usize, c: char) {
+    let mut result = String::from("");
+    result.push_str(&get_input_prefix(app, i));
+    result.push_str(&c.to_string());
+    result.push_str(&get_input_suffix(app, i));
+    app.input = result;
+}
+
+fn remove_char_from_input(app: &mut State, i: usize) -> char {
+    let mut result = String::from("");
+    result.push_str(&get_input_prefix(app, i - 1));
+    result.push_str(&get_input_suffix(app, i));
+    let c = app.input.chars().nth(i - 1).unwrap();
+    app.input = result;
+    c
+}
+
+#[allow(clippy::too_many_lines)]
 async fn key_handler(
-    input: Key,
+    input: TermEvent,
     search_mode: SearchMode,
-    db: &mut (impl Database + Send + Sync),
+    db: &mut impl Database,
     app: &mut State,
 ) -> Option<String> {
     match input {
-        Key::Esc | Key::Ctrl('c' | 'd' | 'g') => return Some(String::from("")),
-        Key::Char('\n') => {
+        TermEvent::Key(Key::Esc | Key::Ctrl('c' | 'd' | 'g')) => return Some(String::from("")),
+        TermEvent::Key(Key::Char('\n')) => {
             let i = app.results_state.selected().unwrap_or(0);
 
             return Some(
@@ -278,7 +324,7 @@ async fn key_handler(
                     .map_or(app.input.clone(), |h| h.command.clone()),
             );
         }
-        Key::Alt(c) if ('1'..='9').contains(&c) => {
+        TermEvent::Key(Key::Alt(c)) if ('1'..='9').contains(&c) => {
             let c = c.to_digit(10)? as usize;
             let i = app.results_state.selected()? + c;
 
@@ -288,32 +334,59 @@ async fn key_handler(
                     .map_or(app.input.clone(), |h| h.command.clone()),
             );
         }
-        Key::Char(c) => {
-            app.input.push(c);
+        TermEvent::Key(Key::Left | Key::Ctrl('h')) => {
+            if app.cursor_index != 0 {
+                app.cursor_index -= 1;
+            }
+        }
+        TermEvent::Key(Key::Right | Key::Ctrl('l')) => {
+            if app.cursor_index < app.input.width() {
+                app.cursor_index += 1;
+            }
+        }
+        TermEvent::Key(Key::Ctrl('a')) => {
+            app.cursor_index = 0;
+        }
+        TermEvent::Key(Key::Ctrl('e')) => {
+            app.cursor_index = app.input.chars().count();
+        }
+        TermEvent::Key(Key::Char(c)) => {
+            insert_char_into_input(app, app.cursor_index, c);
+            app.cursor_index += 1;
             query_results(app, search_mode, db).await.unwrap();
         }
-        Key::Backspace => {
-            app.input.pop();
-            query_results(app, search_mode, db).await.unwrap();
-        }
-        // \u{7f} is escape sequence for backspace
-        Key::Alt('\u{7f}') => {
-            let words: Vec<&str> = app.input.split(' ').collect();
-            if words.is_empty() {
+        TermEvent::Key(Key::Backspace) => {
+            if app.cursor_index == 0 {
                 return None;
             }
-            if words.len() == 1 {
-                app.input = String::from("");
-            } else {
-                app.input = words[0..(words.len() - 1)].join(" ");
+            remove_char_from_input(app, app.cursor_index);
+            app.cursor_index -= 1;
+            query_results(app, search_mode, db).await.unwrap();
+        }
+        TermEvent::Key(Key::Ctrl('w')) => {
+            let mut stop_on_next_whitespace = false;
+            loop {
+                if app.cursor_index == 0 {
+                    break;
+                }
+                if app.input.chars().nth(app.cursor_index - 1) == Some(' ')
+                    && stop_on_next_whitespace
+                {
+                    break;
+                }
+                if !remove_char_from_input(app, app.cursor_index).is_whitespace() {
+                    stop_on_next_whitespace = true;
+                }
+                app.cursor_index -= 1;
             }
             query_results(app, search_mode, db).await.unwrap();
         }
-        Key::Ctrl('u') => {
+        TermEvent::Key(Key::Ctrl('u')) => {
             app.input = String::from("");
+            app.cursor_index = 0;
             query_results(app, search_mode, db).await.unwrap();
         }
-        Key::Ctrl('r') => {
+        TermEvent::Key(Key::Ctrl('r')) => {
             app.filter_mode = match app.filter_mode {
                 FilterMode::Global => FilterMode::Host,
                 FilterMode::Host => FilterMode::Session,
@@ -323,7 +396,8 @@ async fn key_handler(
 
             query_results(app, search_mode, db).await.unwrap();
         }
-        Key::Down | Key::Ctrl('n') => {
+        TermEvent::Key(Key::Down | Key::Ctrl('n' | 'j'))
+        | TermEvent::Mouse(MouseEvent::Press(MouseButton::WheelDown, _, _)) => {
             let i = match app.results_state.selected() {
                 Some(i) => {
                     if i == 0 {
@@ -336,7 +410,8 @@ async fn key_handler(
             };
             app.results_state.select(Some(i));
         }
-        Key::Up | Key::Ctrl('p') => {
+        TermEvent::Key(Key::Up | Key::Ctrl('p' | 'k'))
+        | TermEvent::Mouse(MouseEvent::Press(MouseButton::WheelUp, _, _)) => {
             let i = match app.results_state.selected() {
                 Some(i) => {
                     if i >= app.results.len() - 1 {
@@ -426,9 +501,16 @@ fn draw<T: Backend>(f: &mut Frame<'_, T>, history_count: i64, app: &mut State) {
     );
     f.render_widget(input, chunks[2]);
 
+    let width = UnicodeWidthStr::width(
+        app.input
+            .chars()
+            .take(app.cursor_index)
+            .collect::<String>()
+            .as_str(),
+    );
     f.set_cursor(
         // Put cursor past the end of the input text
-        chunks[2].x + app.input.width() as u16 + 1,
+        chunks[2].x + width as u16 + 1,
         // Move one line down, from the border to the input line
         chunks[2].y + 1,
     );
@@ -498,7 +580,13 @@ fn draw_compact<T: Backend>(f: &mut Frame<'_, T>, history_count: i64, app: &mut
     app.render_results(f, chunks[1], Block::default());
     f.render_widget(input, chunks[2]);
 
-    let extra_width = app.input.width() + filter_mode.len();
+    let extra_width = UnicodeWidthStr::width(
+        app.input
+            .chars()
+            .take(app.cursor_index)
+            .collect::<String>()
+            .as_str(),
+    ) + filter_mode.len();
 
     f.set_cursor(
         // Put cursor past the end of the input text
@@ -517,7 +605,7 @@ async fn select_history(
     search_mode: SearchMode,
     filter_mode: FilterMode,
     style: atuin_client::settings::Style,
-    db: &mut (impl Database + Send + Sync),
+    db: &mut impl Database,
 ) -> Result<String> {
     let stdout = stdout().into_raw_mode()?;
     let stdout = MouseTerminal::from(stdout);
@@ -528,8 +616,12 @@ async fn select_history(
     // Setup event handlers
     let events = Events::new();
 
+    let input = query.join(" ");
+    // Put the cursor at the end of the query by default
+    let cursor_index = input.chars().count();
     let mut app = State {
-        input: query.join(" "),
+        input,
+        cursor_index,
         results: Vec::new(),
         results_state: ListState::default(),
         context: current_context(),
@@ -565,117 +657,99 @@ async fn select_history(
 // This is supposed to more-or-less mirror the command line version, so ofc
 // it is going to have a lot of args
 #[allow(clippy::too_many_arguments)]
-pub async fn run(
+async fn run_non_interactive(
     settings: &Settings,
+    list_mode: ListMode,
     cwd: Option<String>,
     exit: Option<i64>,
-    interactive: bool,
-    human: bool,
     exclude_exit: Option<i64>,
     exclude_cwd: Option<String>,
     before: Option<String>,
     after: Option<String>,
-    cmd_only: bool,
+    limit: Option<i64>,
     query: &[String],
-    db: &mut (impl Database + Send + Sync),
+    db: &mut impl Database,
 ) -> Result<()> {
-    let dir = if let Some(cwd) = cwd {
-        if cwd == "." {
-            let current = std::env::current_dir()?;
-            let current = current.as_os_str();
-            let current = current.to_str().unwrap();
+    let dir = if cwd.as_deref() == Some(".") {
+        let current = std::env::current_dir()?;
+        let current = current.as_os_str();
+        let current = current.to_str().unwrap();
 
-            Some(current.to_owned())
-        } else {
-            Some(cwd)
-        }
+        Some(current.to_owned())
     } else {
-        None
+        cwd
     };
 
-    if interactive {
-        let item = select_history(
-            query,
+    let context = current_context();
+
+    let results = db
+        .search(
+            limit,
             settings.search_mode,
             settings.filter_mode,
-            settings.style,
-            db,
+            &context,
+            query.join(" ").as_str(),
         )
         .await?;
-        eprintln!("{}", item);
-    } else {
-        let context = current_context();
 
-        let results = db
-            .search(
-                None,
-                settings.search_mode,
-                settings.filter_mode,
-                &context,
-                query.join(" ").as_str(),
-            )
-            .await?;
-
-        // TODO: This filtering would be better done in the SQL query, I just
-        // need a nice way of building queries.
-        let results: Vec<History> = results
-            .iter()
-            .filter(|h| {
-                if let Some(exit) = exit {
-                    if h.exit != exit {
-                        return false;
-                    }
+    // TODO: This filtering would be better done in the SQL query, I just
+    // need a nice way of building queries.
+    let results: Vec<History> = results
+        .iter()
+        .filter(|h| {
+            if let Some(exit) = exit {
+                if h.exit != exit {
+                    return false;
                 }
+            }
 
-                if let Some(exit) = exclude_exit {
-                    if h.exit == exit {
-                        return false;
-                    }
+            if let Some(exit) = exclude_exit {
+                if h.exit == exit {
+                    return false;
                 }
+            }
 
-                if let Some(cwd) = &exclude_cwd {
-                    if h.cwd.as_str() == cwd.as_str() {
-                        return false;
-                    }
+            if let Some(cwd) = &exclude_cwd {
+                if h.cwd.as_str() == cwd.as_str() {
+                    return false;
                 }
+            }
 
-                if let Some(cwd) = &dir {
-                    if h.cwd.as_str() != cwd.as_str() {
-                        return false;
-                    }
+            if let Some(cwd) = &dir {
+                if h.cwd.as_str() != cwd.as_str() {
+                    return false;
                 }
+            }
 
-                if let Some(before) = &before {
-                    let before = chrono_english::parse_date_string(
-                        before.as_str(),
-                        Utc::now(),
-                        chrono_english::Dialect::Uk,
-                    );
+            if let Some(before) = &before {
+                let before = chrono_english::parse_date_string(
+                    before.as_str(),
+                    Utc::now(),
+                    chrono_english::Dialect::Uk,
+                );
 
-                    if before.is_err() || h.timestamp.gt(&before.unwrap()) {
-                        return false;
-                    }
+                if before.is_err() || h.timestamp.gt(&before.unwrap()) {
+                    return false;
                 }
+            }
 
-                if let Some(after) = &after {
-                    let after = chrono_english::parse_date_string(
-                        after.as_str(),
-                        Utc::now(),
-                        chrono_english::Dialect::Uk,
-                    );
+            if let Some(after) = &after {
+                let after = chrono_english::parse_date_string(
+                    after.as_str(),
+                    Utc::now(),
+                    chrono_english::Dialect::Uk,
+                );
 
-                    if after.is_err() || h.timestamp.lt(&after.unwrap()) {
-                        return false;
-                    }
+                if after.is_err() || h.timestamp.lt(&after.unwrap()) {
+                    return false;
                 }
+            }
 
-                true
-            })
-            .map(std::borrow::ToOwned::to_owned)
-            .collect();
-
-        super::history::print_list(&results, human, cmd_only);
-    }
+            true
+        })
+        .map(std::borrow::ToOwned::to_owned)
+        .collect();
 
+    super::history::print_list(&results, list_mode);
     Ok(())
 }
