@@ -1,8 +1,7 @@
 #![warn(clippy::pedantic, clippy::nursery)]
 #![allow(clippy::use_self)] // not 100% reliable
 
-use clap::AppSettings;
-use clap::Parser;
+use clap::{AppSettings, Parser};
 use eyre::Result;
 
 #[macro_use]
@@ -26,12 +25,11 @@ struct Atuin {
 }
 
 impl Atuin {
-    async fn run(self) -> Result<()> {
-        self.atuin.run().await
+    fn run(self) -> Result<()> {
+        self.atuin.run()
     }
 }
 
-#[tokio::main]
-async fn main() -> Result<()> {
-    Atuin::parse().run().await
+fn main() -> Result<()> {
+    Atuin::parse().run()
 }
